@@ -31,6 +31,7 @@ struct MHINFO {
     bool andOthers;
     bool doAttach;
     bool atimage;
+    bool pgp;
     uchar error64;
     bool ne;			/* non english */
 };
@@ -208,6 +209,8 @@ writeAttachment(struct MHINFO *w)
     const char *atname;
     if((ismc | ignoreImages) && w->atimage)
 	return;			/* image ignored */
+    if(w->pgp)
+	return;			/* Ignore PGP signatures. */
     if(w->error64 == BAD64)
 	i_printf(MSG_Abbreviated);
     if(w->start == w->end) {
@@ -271,7 +274,47 @@ writeAttachments(struct MHINFO *w)
     }
 }				/* writeAttachments */
 
-void
+/* find the last mail in the unread directory */
+static int unreadMax, unreadMin, unreadCount;
+static int unreadBase;		/* find min larger than base */
+
+static void
+unreadStats(void)
+{
+    const char *f;
+    int n;
+
+    unreadMax = 0;
+    unreadMin = 0;
+    unreadCount = 0;
+
+    while(f = nextScanFile(mailUnread)) {
+	if(!stringIsNum(f))
+	    continue;
+	n = atoi(f);
+	if(n > unreadMax)
+	    unreadMax = n;
+	if(n > unreadBase) {
+	    if(!unreadMin || n < unreadMin)
+		unreadMin = n;
+	    ++unreadCount;
+	}
+    }
+}				/* unreadStats */
+
+static char *umf;		/* unread mail file */
+static char *umf_end;
+static int umfd;		/* file descriptor for the above */
+/* string to hold the mail message, and then its length */
+static char *mailstring;
+static int mailstring_l;
+/* convert mail message to/from utf8 if need be. */
+/* This isn't really right, cause it should be done per mime component. */
+static char *mailu8;
+static int mailu8_l;
+
+/* Returns number of messages fetched */
+int
 fetchMail(int account)
 {
     const struct MACCOUNT *a = accounts + account - 1;
@@ -279,367 +322,477 @@ fetchMail(int account)
     const char *pass = a->password;
     const char *host = a->inurl;
     int nmsgs, m, j, k;
-
-    if(!isInteractive) {
-	if(!zapMail)
-	    i_printfExit(MSG_FetchNotBackgnd);
-    }
+    int nfetch = 0;		/* number of messages actually fetched */
 
     if(!mailDir)
 	i_printfExit(MSG_NoMailDir);
     if(chdir(mailDir))
 	i_printfExit(MSG_NoDirChange, mailDir);
 
-    if(!loadAddressBook())
-	showErrorAbort();
+    if(!umf) {
+	umf = allocMem(strlen(mailUnread) + 12);
+	sprintf(umf, "%s/", mailUnread);
+	umf_end = umf + strlen(umf);
+    }
+    unreadBase = 0;
+    unreadStats();
 
     if(!mailConnect(host, a->inport, a->inssl))
 	showErrorAbort();
     if(!mailGetLine())
 	showErrorAbort();
-    if(memcmp(serverLine, "+OK ", 4))
-	i_printfExit(MSG_BadPopIntro, serverLine);
+    if(memcmp(serverLine, "+OK ", 4)) {
+	i_printf(MSG_BadPopIntro, serverLine);
+	mailClose();
+	return nfetch;
+    }
     sprintf(serverLine, "user %s%s", login, eol);
     mailPutGetError(serverLine);
     if(pass) {			/* I think this is always required */
 	sprintf(serverLine, "pass %s%s", pass, eol);
 	mailPutGetError(serverLine);
     }				/* password */
-    if(memcmp(serverLine, "+OK", 3))
-	i_printfExit(MSG_PopNotComplete, serverLine);
+    if(memcmp(serverLine, "+OK", 3)) {
+	i_printf(MSG_PopNotComplete, serverLine);
+	mailClose();
+	return nfetch;
+    }
 
 /* How many mail messages? */
     mailPutGetError("stat\r\n");
-    if(memcmp(serverLine, "+OK ", 4))
-	i_printfExit(MSG_NoStatusMailBox, serverLine);
+    if(memcmp(serverLine, "+OK ", 4)) {
+	i_printf(MSG_NoStatusMailBox, serverLine);
+	mailClose();
+	return nfetch;
+    }
     nmsgs = atoi(serverLine + 4);
     if(!nmsgs) {
-	i_puts(MSG_NoMail);
 	mailClose();
+	return nfetch;
+    }
+
+    for(m = 1; m <= nmsgs; ++m) {
+	char retrbuf[5000];
+	bool retr1;
+
+/* Grab the message */
+	sprintf(serverLine, "retr %d%s", m, eol);
+	if(!mailPutLine(serverLine, false))
+	    showErrorAbort();
+
+	mailstring = initString(&mailstring_l);
+	retr1 = true;
+	while(true) {
+	    int nr;
+	    if(a->inssl)
+		nr = ssl_read(retrbuf, sizeof (retrbuf));
+	    else
+		nr = tcp_read(mssock, retrbuf, sizeof (retrbuf));
+	    if(nr <= 0) {
+		i_printf(MSG_ErrorReadMess, errno);
+		mailClose();
+		nzFree(mailstring);
+		return nfetch;
+	    }
+
+	    if(retr1) {
+/* add null, to make it easy to print the error message if necessary */
+		if(nr < sizeof (retrbuf))
+		    retrbuf[nr] = 0;
+		if(memcmp(retrbuf, "+OK", 3)) {
+		    i_printf(MSG_ErrorFetchMess, m, retrbuf);
+		    mailClose();
+		    nzFree(mailstring);
+		    return nfetch;
+		}
+
+		j = 3;		/* skip past ok */
+		while(retrbuf[j] != '\n')
+		    ++j;
+		++j;
+		nr -= j;
+		memmove(retrbuf, retrbuf + j, nr);
+		retr1 = false;
+	    }
+
+	    if(nr)
+		stringAndBytes(&mailstring, &mailstring_l, retrbuf, nr);
+
+/* . by itself on a line ends the transmission */
+	    j = mailstring_l - 1;
+	    if(j < 0)
+		continue;
+	    if(mailstring[j] != '\n')
+		continue;
+	    --j;
+	    if(j >= 0 && mailstring[j] == '\r')
+		--j;
+	    if(j < 0)
+		continue;
+	    if(mailstring[j] != '.')
+		continue;
+	    if(!j)
+		break;
+	    if(mailstring[j - 1] == '\n')
+		break;
+	}
+	mailstring_l = j;
+
+/* get rid of the dos returns, and dot strip */
+	for(j = k = 0; j < mailstring_l; ++j) {
+	    if(!j && mailstring[j] == '.')
+		continue;
+	    if(j && mailstring[j] == '.' && mailstring[j - 1] == '\n')
+		continue;
+	    if(mailstring[j] == '\r' && j < mailstring_l - 1 &&
+	       mailstring[j + 1] == '\n')
+		continue;
+	    mailstring[k++] = mailstring[j];
+	}
+	mailstring_l = k;
+	mailstring[k] = 0;
+
+/* got the file, save it in unread */
+	sprintf(umf_end, "%d", unreadMax + m);
+	umfd = open(umf, O_WRONLY | O_TEXT | O_CREAT, 0666);
+	if(umfd < 0)
+	    i_printfExit(MSG_NoCreate, umf);
+	if(write(umfd, mailstring, mailstring_l) < mailstring_l)
+	    i_printfExit(MSG_NoWrite, umf);
+	close(umfd);
+	++nfetch;
+	nzFree(mailstring);
+	mailstring = 0;
+
+	sprintf(serverLine, "dele %d%s", m, eol);
+	if(!mailPutLine(serverLine, false))
+	    showErrorAbort();
+	if(!mailGetLine())
+	    i_printfExit(MSG_MailTimeOver);
+	if(memcmp(serverLine, "+OK", 3))
+	    i_printfExit(MSG_UnableDelMail, serverLine);
+    }				/* loop over mail messages */
+
+    mailClose();
+    return nfetch;
+}				/* fetchMail */
+
+void
+scanMail(void)
+{
+    int nmsgs, m, j, k;
+
+    if(!isInteractive)
+	i_printfExit(MSG_FetchNotBackgnd);
+    if(!mailDir)
+	i_printfExit(MSG_NoMailDir);
+    if(chdir(mailDir))
+	i_printfExit(MSG_NoDirChange, mailDir);
+
+    if(!umf) {
+	umf = allocMem(strlen(mailUnread) + 12);
+	sprintf(umf, "%s/", mailUnread);
+	umf_end = umf + strlen(umf);
+    }
+
+/* How many mail messages? */
+    unreadBase = 0;
+    unreadStats();
+    nmsgs = unreadCount;
+    if(!nmsgs) {
+	i_puts(MSG_NoMail);
 	exit(0);
     }
-
-    if(!(zapMail | passMail)) {
-/* We look up a lot of hosts; make a tcp connect */
-	sethostent(1);
-/* the conect will drop when the program exits */
-    }
-
     i_printf(MSG_MessagesX, nmsgs);
-    if(zapMail && nmsgs > 300)
-	nmsgs = 300;
+
+    loadAddressBook();
 
     for(m = 1; m <= nmsgs; ++m) {
 	const char *redirect = 0;	/* send mail elsewhere */
 	char key;
 	const char *atname;	/* name of attachment */
 	bool delflag = false;	/* delete this mail */
-	int exact_l;
-	char *exact = initString(&exact_l);	/* the exact message */
-	int exactf_l;
-	char *exactf = 0;	/* utf8 formatted */
 	int displine;
+	int stashNumber = -1;
 
-	if(zapMail)
-	    delflag = true;
-	else {			/* read the next message */
-	    char retrbuf[5000];
-	    bool retr1;
 /* We need to clear out the editor, from the last message,
  * then read in this one, in its entirety.
  * This is probably a waste, since the user might delete a megabyte message
  * after seeing only the first paragraph, or even the subject,
  * but as for programming, it's easier to read the whole thing in right now. */
-	    if(lastMailInfo)
-		freeMailInfo(lastMailInfo);
-	    lastMailInfo = 0;
-	    nzFree(lastMailText);
-	    lastMailText = 0;
-	    if(sessionList[1].lw)
-		cxQuit(1, 2);
-	    cs = 0;
-	    linesReset();
-	    cxSwitch(1, false);
+
+	if(lastMailInfo)
+	    freeMailInfo(lastMailInfo);
+	lastMailInfo = 0;
+	nzFree(lastMailText);
+	lastMailText = 0;
+
+	if(sessionList[1].lw)
+	    cxQuit(1, 2);
+	cs = 0;
+	linesReset();
+	cxSwitch(1, false);
+
 /* Now grab the entire message */
-	    sprintf(serverLine, "retr %d%s", m, eol);
-	    if(!mailPutLine(serverLine, false))
+	unreadStats();
+	sprintf(umf_end, "%d", unreadMin);
+	if(!fileIntoMemory(umf, &mailstring, &mailstring_l))
+	    showErrorAbort();
+	unreadBase = unreadMin;
+
+	iuReformat(mailstring, mailstring_l, &mailu8, &mailu8_l);
+
+	if(mailu8) {
+	    if(!addTextToBuffer((pst) mailu8, mailu8_l, 0, false))
 		showErrorAbort();
-	    retr1 = true;
-	    while(true) {
-		int nr;
-		if(a->inssl)
-		    nr = ssl_read(retrbuf, sizeof (retrbuf));
-		else
-		    nr = tcp_read(mssock, retrbuf, sizeof (retrbuf));
-		if(nr <= 0)
-		    i_printfExit(MSG_ErrorReadMess, errno);
-		if(retr1) {
-/* add null, to make it easy to print the error message, if necessary */
-		    if(nr < sizeof (retrbuf))
-			retrbuf[nr] = 0;
-		    if(memcmp(retrbuf, "+OK", 3))
-			i_printfExit(MSG_ErrorFetchMess, m, retrbuf);
-		    j = 3;
-		    while(retrbuf[j] != '\n')
-			++j;
-		    ++j;
-		    nr -= j;
-		    memcpy(retrbuf, retrbuf + j, nr);
-		    retr1 = false;
-		}
-		if(nr)
-		    stringAndBytes(&exact, &exact_l, retrbuf, nr);
-/* . by itself on a line ends the transmission */
-		j = exact_l - 1;
-		if(j < 0)
-		    continue;
-		if(exact[j] != '\n')
-		    continue;
-		--j;
-		if(j >= 0 && exact[j] == '\r')
-		    --j;
-		if(j < 0)
-		    continue;
-		if(exact[j] != '.')
-		    continue;
-		if(!j)
-		    break;
-		if(exact[j - 1] == '\n')
-		    break;
-	    }
-	    exact_l = j;
+	} else {
+	    if(!addTextToBuffer((pst) mailstring, mailstring_l, 0, false))
+		showErrorAbort();
+	}
 
-/* get rid of the dos returns, and dot strip */
-	    for(j = k = 0; j < exact_l; ++j) {
-		if(!j && exact[j] == '.')
-		    continue;
-		if(j && exact[j] == '.' && exact[j - 1] == '\n')
-		    continue;
-		if(exact[j] == '\r' && j < exact_l - 1 && exact[j + 1] == '\n')
-		    continue;
-		exact[k++] = exact[j];
-	    }
-	    exact_l = k;
-	    exact[k] = 0;
+	browseCurrentBuffer();
 
-	    iuReformat(exact, exact_l, &exactf, &exactf_l);
+	if(!passMail) {
+	    mailIsBlack = onBlacklist();
+	    redirect = mailRedirect(lastMailInfo->to,
+	       lastMailInfo->from, lastMailInfo->reply, lastMailInfo->subject);
+	}
 
-	    if(exactf) {
-		if(!addTextToBuffer((pst) exactf, exactf_l, 0, false))
-		    showErrorAbort();
-	    } else {
-		if(!addTextToBuffer((pst) exact, exact_l, 0, false))
-		    showErrorAbort();
+	if(redirect) {
+	    delflag = true;
+	    key = 'w';
+	    if(*redirect == '-')
+		++redirect, key = 'u';
+	    if(stringEqual(redirect, "x"))
+		i_puts(MSG_Junk);
+	    else
+		printf("> %s\n", redirect);
+	} else if((mailIsSpam | mailIsBlack) && spamCan) {
+	    redirect = spamCan;
+	    key = 'u';
+	    i_printf(MSG_Spam);
+	    if(lastMailInfo->from[0]) {
+		i_printf(MSG_From);
+		printf("%s", lastMailInfo->from);
+	    } else if(lastMailInfo->reply[0]) {
+		i_printf(MSG_From);
+		printf("%s", lastMailInfo->reply);
 	    }
-
-	    if(!unformatMail) {
-		browseCurrentBuffer();
-		if(!passMail) {
-		    mailIsBlack = onBlacklist();
-		    redirect = mailRedirect(lastMailInfo->to,
-		       lastMailInfo->from, lastMailInfo->reply,
-		       lastMailInfo->subject);
-		}
-		if(redirect) {
-		    key = 'w';
-		    if(*redirect == '-')
-			++redirect, key = 'u';
-		    if(stringEqual(redirect, "x"))
-			i_puts(MSG_Junk);
-		    else
-			printf("> %s\n", redirect);
-		} else if((mailIsSpam | mailIsBlack) && spamCan) {
-		    redirect = spamCan;
-		    key = 'u';
-		    i_printf(MSG_Spam);
-		    if(lastMailInfo->from[0]) {
-			i_printf(MSG_From);
-			printf("%s", lastMailInfo->from);
-		    } else if(lastMailInfo->reply[0]) {
-			i_printf(MSG_From);
-			printf("%s", lastMailInfo->reply);
-		    }
-		    nl();
-		} else if(!nattach &&	/* drop empty mail message */
-		   cw->dol -
-		   (strlen(lastMailInfo->subject) != 0) -
-		   (strlen(lastMailInfo->from) != 0) -
-		   (strlen(lastMailInfo->reply) != 0) <= 1) {
-		    redirect = "x";
-		    i_puts(MSG_Empty);
-		}
-	    }
-	    if(redirect)
-		delflag = true;
+	    nl();
+	}
 
 /* display the next page of mail and get a command from the keyboard */
-	    displine = 1;
-	    while(true) {
-		if(!delflag) {	/* show next page */
-		  nextpage:
-		    if(displine <= cw->dol) {
-			for(j = 0; j < 20 && displine <= cw->dol;
-			   ++j, ++displine) {
-			    char *showline = (char *)fetchLine(displine, 1);
-			    k = pstLength((pst) showline);
-			    showline[--k] = 0;
-			    printf("%s\n", showline);
-			    nzFree(showline);
-			}
+	displine = 1;
+	while(true) {
+	    if(!delflag) {	/* show next page */
+	      nextpage:
+		if(displine <= cw->dol) {
+		    for(j = 0; j < 20 && displine <= cw->dol; ++j, ++displine) {
+			char *showline = (char *)fetchLine(displine, 1);
+			k = pstLength((pst) showline);
+			showline[--k] = 0;
+			printf("%s\n", showline);
+			nzFree(showline);
 		    }
 		}
+	    }
 
 /* get key command */
-		while(true) {
-		    if(!delflag) {
+	    while(true) {
+		if(!delflag) {
 /* interactive prompt depends on whether there is more text or not */
-			printf("%c ", displine > cw->dol ? '?' : '*');
-			fflush(stdout);
-			key = getLetter("qx? nwkudijJ");
-			printf("\b\b\b");
-			fflush(stdout);
+		    printf("%c ", displine > cw->dol ? '?' : '*');
+		    fflush(stdout);
+		    key = getLetter("q? nwudijJ");
+		    printf("\b\b\b");
+		    fflush(stdout);
 
-			switch (key) {
-			case 'q':
-			    i_puts(MSG_Quit);
-			    mailClose();
-			case 'x':
-			    exit(0);
-			case 'n':
-			    i_puts(MSG_Next);
-			    goto afterinput;
-			case 'd':
-			    i_puts(MSG_Delete);
-			    delflag = true;
-			    goto afterinput;
-			case 'i':
-			    i_puts(MSG_IPDelete);
-			    if(!cw->iplist || cw->iplist[0] == -1) {
-				if(passMail)
-				    i_puts(MSG_POption);
-				else
-				    i_puts(ipbFile ? MSG_None :
-				       MSG_NoBlackList);
-			    } else {
-				IP32bit addr;
-				for(k = 0; (addr = cw->iplist[k]) != NULL_IP;
-				   ++k) {
-				    puts(tcp_ip_dots(addr));
-				    if(nipblack == MAXIPBLACK)
-					continue;
-				    ipblacklist[nipblack] = addr;
-				    ipblackmask[nipblack] = 0xffffffff;
-				    ipblackcomp[nipblack] = false;
-				    ++nipblack;
-				}
-			    }
-			    delflag = true;
-			    goto afterinput;
-			case 'j':
-			case 'J':
-			    i_puts(MSG_Junk);
-			    if(!junkSubject(lastMailInfo->subject, key))
-				continue;
-			    delflag = true;
-			    goto afterinput;
-			case ' ':
-			    if(displine > cw->dol)
-				i_puts(MSG_EndMessage);
-			    goto nextpage;
-			case '?':
-			    i_puts(MSG_MailHelp);
-			    continue;
-			case 'k':
-			case 'w':
-			case 'u':
-			    break;
-			default:
-			    i_puts(MSG_NYI);
-			    continue;
-			}	/* switch */
-		    }
+		    switch (key) {
+		    case 'q':
+			i_puts(MSG_Quit);
+			exit(0);
 
-		    /* delflag or not */
-		    /* At this point we're saving the mail somewhere. */
-		    if(key != 'k')
+		    case 'n':
+			i_puts(MSG_Next);
+			goto afterinput;
+
+		    case 'd':
+			i_puts(MSG_Delete);
 			delflag = true;
-		    atname = redirect;
-		  savemail:
-		    if(!atname)
-			atname = getFileName(0, false);
-		    if(!stringEqual(atname, "x")) {
-			char exists = fileTypeByName(atname, false);
-			int fsize;	/* file size */
-			int fh =
-			   open(atname, O_WRONLY | O_TEXT | O_CREAT | O_APPEND,
-			   0666);
-			if(fh < 0) {
-			    i_printf(MSG_NoCreate, atname);
+			goto afterinput;
+
+		    case 'i':
+			i_puts(MSG_IPDelete);
+			if(!cw->iplist || cw->iplist[0] == -1) {
+			    if(passMail)
+				i_puts(MSG_POption);
+			    else
+				i_puts(ipbFile ? MSG_None : MSG_NoBlackList);
+			} else {
+			    IP32bit addr;
+			    for(k = 0; (addr = cw->iplist[k]) != NULL_IP; ++k) {
+				puts(tcp_ip_dots(addr));
+				if(nipblack == MAXIPBLACK)
+				    continue;
+				ipblacklist[nipblack] = addr;
+				ipblackmask[nipblack] = 0xffffffff;
+				ipblackcomp[nipblack] = false;
+				++nipblack;
+			    }
+			}
+			delflag = true;
+			goto afterinput;
+
+		    case 'j':
+		    case 'J':
+			i_puts(MSG_Junk);
+			if(!junkSubject(lastMailInfo->subject, key))
+			    continue;
+			delflag = true;
+			goto afterinput;
+
+		    case ' ':
+			if(displine > cw->dol)
+			    i_puts(MSG_EndMessage);
+			goto nextpage;
+
+		    case '?':
+			i_puts(MSG_MailHelp);
+			continue;
+
+		    case 'w':
+		    case 'u':
+			break;
+
+		    default:
+			i_puts(MSG_NYI);
+			continue;
+		    }		/* switch */
+		}
+
+		/* At this point we're saving the mail somewhere. */
+		delflag = true;
+		atname = redirect;
+
+	      savemail:
+		if(!atname)
+		    atname = getFileName(0, false);
+		if(!stringEqual(atname, "x")) {
+		    char exists = fileTypeByName(atname, false);
+		    int fsize;	/* file size */
+		    int fh =
+		       open(atname, O_WRONLY | O_TEXT | O_CREAT | O_APPEND,
+		       0666);
+		    if(fh < 0) {
+			i_printf(MSG_NoCreate, atname);
+			goto savemail;
+		    }
+		    if(exists)
+			write(fh,
+			   "======================================================================\n",
+			   71);
+		    if(key == 'u') {
+			if(write(fh, mailstring, mailstring_l) < mailstring_l) {
+			  badsave:
+			    i_printf(MSG_NoWrite, atname);
+			    close(fh);
 			    goto savemail;
 			}
-			if(exists)
-			    write(fh,
-			       "======================================================================\n",
-			       71);
-			if(key == 'u' || unformatMail) {
-			    if(write(fh, exact, exact_l) < exact_l) {
-			      badsave:
-				i_printf(MSG_NoWrite, atname);
-				close(fh);
-				goto savemail;
+			close(fh);
+			fsize = mailstring_l;
+		    } else {
+
+			if(mailStash) {
+			    char *rmf;	/* raw mail file */
+			    int rmfh;	/* file handle to same */
+/* I want a fairly easy filename, in case I want to go look at the original.
+ * Not a 30 character message ID that I am forced to cut&paste.
+ * 4 or 5 digits would be nice.
+ * So the filename looks like /home/foo/.Trash/rawmail/36921
+ * I pick the digits randomly.
+ * Please don't accumulate 100,000 emails before you empty your trash.
+ * It's good to have a cron job empty the trash early Sunday morning.
+*/
+
+			    k = strlen(mailStash);
+			    rmf = allocMem(k + 12);
+/* Try 20 times, then give up. */
+			    for(j = 0; j < 20; ++j) {
+				int rn = rand() % 100000;	/* random number */
+				sprintf(rmf, "%s/%05d", mailStash, rn);
+				if(fileTypeByName(rmf, false))
+				    continue;
+/* dump the original mail into the file */
+				rmfh =
+				   open(rmf,
+				   O_WRONLY | O_TEXT | O_CREAT | O_APPEND,
+				   0666);
+				if(rmfh < 0)
+				    break;
+				if(write(rmfh, mailstring,
+				   mailstring_l) < mailstring_l) {
+				    close(rmfh);
+				    unlink(rmf);
+				    break;
+				}
+				close(rmfh);
+/* written successfully, remember the stash number */
+				stashNumber = rn;
+				break;
 			    }
-			    close(fh);
-			    fsize = exact_l;
-			} else {
-			    fsize = 0;
-			    for(j = 1; j <= cw->dol; ++j) {
-				char *showline = (char *)fetchLine(j, 1);
-				int len = pstLength((pst) showline);
-				if(write(fh, showline, len) < len)
-				    goto badsave;
-				nzFree(showline);
-				fsize += len;
-			    }	/* loop over lines */
-			    close(fh);
-			    if(nattach)
-				writeAttachments(lastMailInfo);
-			}	/* unformat or format */
-			if(atname != spamCan) {
-			    i_printf(MSG_MailSaved, fsize);
-			    if(exists)
-				i_printf(MSG_Appended);
-			    nl();
 			}
-		    }		/* saving to a real file */
-		    goto afterinput;
 
-		}		/* key commands */
-	    }			/* paging through the message */
-	}			/* interactive or zap */
+			fsize = 0;
+			for(j = 1; j <= cw->dol; ++j) {
+			    char *showline = (char *)fetchLine(j, 1);
+			    int len = pstLength((pst) showline);
+			    if(write(fh, showline, len) < len)
+				goto badsave;
+			    nzFree(showline);
+			    fsize += len;
+			}	/* loop over lines */
+
+			if(stashNumber >= 0) {
+			    char addstash[60];
+			    sprintf(addstash, "\nUnformatted %05d\n",
+			       stashNumber);
+			    k = strlen(addstash);
+			    if(write(fh, addstash, k) < k)
+				goto badsave;
+			    fsize += k;
+			}
+
+			close(fh);
+
+			if(nattach)
+			    writeAttachments(lastMailInfo);
+		    }		/* unformat or format */
+
+/* print "mail saved" message */
+		    if(atname != spamCan) {
+			i_printf(MSG_MailSaved, fsize);
+			if(exists)
+			    i_printf(MSG_Appended);
+			nl();
+		    }
+		}		/* saving to a real file */
+		goto afterinput;
+
+	    }			/* key commands */
+	}			/* paging through the message */
+
       afterinput:
+	if(delflag)
+	    unlink(umf);
 
-	if(delflag) {
-/* Remember, it isn't really gone until you quit the session.
- * So if you didn't mean to delete, type x to exit abruptly,
- * then fetch your mail again. */
-	    sprintf(serverLine, "dele %d%s", m, eol);
-	    if(!mailPutLine(serverLine, false))
-		showErrorAbort();
-	    if(!mailGetLine())
-		i_printfExit(MSG_MailTimeOver);
-	    if(memcmp(serverLine, "+OK", 3))
-		i_printfExit(MSG_UnableDelMail, serverLine);
-	}
-	/* deleted */
-	nzFree(exact);
-	nzFree(exactf);
+	nzFree(mailstring);
+	mailstring = 0;
+	nzFree(mailu8);
+	mailu8 = 0;
     }				/* loop over mail messages */
 
-    if(zapMail)
-	printf("%d\n", nmsgs);
-    mailClose();
     exit(0);
-}				/* fetchMail */
+}				/* scanMail */
 
 /* Here are the common keywords for mail header lines.
  * These are in alphabetical order, so you can stick more in as you find them.
@@ -876,7 +1029,7 @@ unpackUploadedFile(const char *post, const char *boundary,
 	b1[0] = b1[1] = ' ';
 	b3 = b2 - 4;
 	b4 = unpack64inline(b1, b3);
-	strcpy(b4, b3);
+	strmove(b4, b3);
 	b2 = b4 + 4;
     }
 
@@ -1034,7 +1187,7 @@ isoDecode(char *vl, char **vrp)
     start = t;
     len = vr - s;
     if(len)
-	memcpy(t, s, len);
+	memmove(t, s, len);
     vr = t + len;
     goto restart;
 
@@ -1071,7 +1224,7 @@ extractLessGreater(char *s)
     vr = strchr(s, '>');
     if(vl && vr && vl < vr) {
 	*vr = 0;
-	strcpy(s, vl + 1);
+	strmove(s, vl + 1);
     }
 }				/* extractLessGreater */
 
@@ -1157,7 +1310,7 @@ headerGlean(char *start, char *end)
 		++j;
 		while(q + j < vr && q[j] == ' ')
 		    ++j;
-		memcpy(q, q + j, vr - q - j);
+		memmove(q, q + j, vr - q - j);
 		vr -= j;
 		--q;		/* try again */
 	    }
@@ -1266,6 +1419,8 @@ headerGlean(char *start, char *end)
 
 	if(memEqualCI(s, "content-type:", q - s)) {
 	    linetype = 'c';
+	    if(memEqualCI(vl, "application/pgp-signature", 25))
+		w->pgp = true;
 	    if(memEqualCI(vl, "text", 4))
 		w->ct = CT_RICH;
 	    if(memEqualCI(vl, "text/html", 9))
@@ -1314,7 +1469,7 @@ headerGlean(char *start, char *end)
     if(!w->reply[0])
 	strcpy(w->reply, w->from);
     if(w->from[0] == '"') {
-	strcpy(w->from, w->from + 1);
+	strmove(w->from, w->from + 1);
 	q = strchr(w->from, '"');
 	if(q)
 	    *q = 0;
@@ -1892,7 +2047,7 @@ setupReply(bool all)
 	if(strchr("srv", linetype[j]))
 	    continue;
 	delText(j, j);
-	strcpy(linetype + j, linetype + j + 1);
+	strmove(linetype + j, linetype + j + 1);
     }
 
 /* move reply to 1, if it isn't already there */
@@ -1923,7 +2078,6 @@ setupReply(bool all)
     }
 
     if(!cw->mailInfo) {
-	cw->browseMode = false;
 	if(all) {
 	    setError(MSG_ReNoInfo);
 	    return false;
@@ -1975,6 +2129,5 @@ setupReply(bool all)
     if(j)
 	rc = addTextToBuffer((unsigned char *)out, j, 1, false);
     nzFree(out);
-    cw->browseMode = false;
     return rc;
 }				/* setupReply */
