@@ -306,7 +306,7 @@ htmlAttrVal(const char *e, const char *name)
  * Are we really suppose to do this? */
     for(b = a; *b == ' '; b++) ;
     if(b > a)
-	strcpy(a, b);
+	strmove(a, b);
     for(b = a + strlen(a) - 1; b >= a && *b == ' '; b--)
 	*b = 0;
     return a;
@@ -492,8 +492,8 @@ anchorSwap(char *buf)
 	    if(w && a && !premode &&
 	       ((state_braces & !state_atext) ||
 	       ((!state_braces) & !state_text))) {
-		memcpy(a, w, s - w);
-		memcpy(a + (s - w), tag, n);
+		memmove(a, w, s - w);
+		memmove(a + (s - w), tag, n);
 		change = true;
 		w = 0;
 	    }
@@ -636,7 +636,7 @@ anchorSwap(char *buf)
 	if(*a != close)
 	    goto putc;
 	++s;
-	memcpy(w, s, a - s);
+	memmove(w, s, a - s);
 	w += a - s;
 	s = a;
 	ss = 0;
@@ -1177,7 +1177,7 @@ andTranslate(const char *s, bool invisible)
     int l, n, j;
     uchar c, d;
     uchar alnum = 0;		/* was last char an alphanumeric */
-    bool premode;
+    bool premode = false;
     char andbuf[16];
 
     static const char *const andwords[] = {
@@ -1442,7 +1442,7 @@ andTranslate(const char *s, bool invisible)
 /* remove leading zeros */
 	if(andbuf[0] == '#')
 	    while(andbuf[1] == '0')
-		strcpy(andbuf + 1, andbuf + 2);
+		strmove(andbuf + 1, andbuf + 2);
 
       lookup:
 	debugPrint(6, "meta %s", andbuf);
@@ -1596,7 +1596,7 @@ cutDuplicateEmail(char *line, const char *dup, int duplen)
 	    return;		/* should never happen */
 	if(duplen == s - line && memEqualCI(line, dup, duplen)) {
 	    ++s;
-	    strcpy(line, s);
+	    strmove(line, s);
 	    continue;
 	}
 	line = s + 1;
