@@ -10,7 +10,6 @@
 
 #include "eb.h"
 
-#include "jsapi.h"
 /* jsprf.h is not publically visible on some systems,
 so I can't #include it here.
 Instead, I'll declare the needed prototype myself, and hope it is consistent
@@ -22,11 +21,11 @@ JS_smprintf(const char *fmt, ...);
 #define PROP_FIXED (JSPROP_ENUMERATE|JSPROP_READONLY|JSPROP_PERMANENT)
 
 
-void *jcx;			/* really JSContext */
-void *jwin;			/* window object, really JSObject */
-void *jdoc;			/* window.document, really JSObject */
-void *jwloc;			/* window.location, really JSObject */
-void *jdloc;			/* document.location, really JSObject */
+JSContext *jcx;			/* really JSContext */
+JSObject *jwin;			/* window object, really JSObject */
+JSObject *jdoc;			/* window.document, really JSObject */
+JSObject *jwloc;		/* window.location, really JSObject */
+JSObject *jdloc;		/* document.location, really JSObject */
 static size_t gStackChunkSize = 8192;
 static FILE *gOutFile, *gErrFile;
 static const char *emptyParms[] = { 0 };
@@ -81,6 +80,69 @@ my_ErrorReporter(JSContext * cx, const char *message, JSErrorReport * report)
     report->flags = 0;
 }				/* my_ErrorReporter */
 
+JSString *
+our_JS_NewStringCopyN(JSContext * cx, const char *s, size_t n)
+{
+    char *converted = NULL;
+    int converted_l = 0;
+    const char *outbytes = s;
+    int outbytes_l = n;
+    JSString *forSpidermonkey = NULL;
+
+    if(!JS_CStringsAreUTF8())
+/* Fixme this is too simple.  We need to decode UTF8 to JSCHAR, for proper
+ * unicode handling.  E.G., JS C strings are not UTF8, but the user has
+ * a UTF8 locale. */
+	return JS_NewStringCopyN(jcx, s, n);
+
+    if(!cons_utf8) {
+/* The string should not be UTF8, because of edbrowse's conversion. */
+	iso2utf(s, n, &converted, &converted_l);
+	outbytes = converted;
+	outbytes_l = converted_l;
+    }
+
+    forSpidermonkey = JS_NewStringCopyN(jcx, outbytes, outbytes_l);
+    nzFree(converted);
+
+    return forSpidermonkey;
+}				/* our_JS_NewStringCopyN */
+JSString *
+our_JS_NewStringCopyZ(JSContext * cx, const char *s)
+{
+    size_t len = strlen(s);
+    return our_JS_NewStringCopyN(jcx, s, len);
+}				/* our_JS_NewStringCopyZ */
+
+char *
+our_JSEncodeString(JSString * str)
+{
+    size_t encodedLength = JS_GetStringEncodingLength(jcx, str);
+    char *buffer = allocMem(encodedLength + 1);
+    buffer[encodedLength] = '\0';
+    size_t result = JS_EncodeStringToBuffer(str, buffer, encodedLength);
+    if(result == (size_t) - 1)
+	i_printfExit(MSG_JSFailure);
+    return buffer;
+}				/* our_JSEncodeString */
+
+char *
+transcode_get_js_bytes(JSString * s)
+{
+    char *converted = NULL;
+    int converted_l = 0;
+    char *origbytes = our_JSEncodeString(s);
+
+    if(!JS_CStringsAreUTF8())
+	return origbytes;
+
+    if(cons_utf8)
+	return origbytes;
+
+    utf2iso(origbytes, strlen(origbytes), &converted, &converted_l);
+    nzFree(origbytes);
+    return converted;
+}				/* our_JS_GetTranscodedBytes */
 
 /*********************************************************************
 When an element is created without a name, it is not linked to its
@@ -107,86 +169,94 @@ Start with window and document.
 
 static JSClass window_class = {
     "Window",
-    JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JSCLASS_HAS_PRIVATE | JSCLASS_GLOBAL_FLAGS,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
 };
 
 static JSBool
-window_ctor(JSContext * cx, JSObject * obj, uintN argc, jsval * argv,
-   jsval * rval)
+window_ctor(JSContext * cx, uintN argc, jsval * vp)
 {
-    const char *newloc = 0;
+    char *newloc = 0;
     const char *winname = 0;
     JSString *str;
+    jsval *argv = JS_ARGV(cx, vp);
+    JSObject *newwin = JS_NewObjectForConstructor(cx, vp);
     if(argc > 0 && (str = JS_ValueToString(jcx, argv[0]))) {
-	newloc = JS_GetStringBytes(str);
+	newloc = our_JSEncodeString(str);
     }
     if(argc > 1 && (str = JS_ValueToString(jcx, argv[1]))) {
-	winname = JS_GetStringBytes(str);
+	winname = transcode_get_js_bytes(str);
     }
 /* third argument is attributes, like window size and location, that we don't care about. */
     javaOpensWindow(newloc, winname);
+    if(newloc)
+	nzFree(newloc);
+
     if(!parsePage)
 	return JS_FALSE;
-    establish_property_object(obj, "opener", jwin);
+    establish_property_object(newwin, "opener", jwin);
+    JS_SET_RVAL(cx, vp, OBJECT_TO_JSVAL(newwin));
     return JS_TRUE;
 }				/* window_ctor */
 
 /* window.open() instantiates a new window object */
 static JSBool
-win_open(JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * rval)
+win_open(JSContext * cx, uintN argc, jsval * vp)
 {
+    jsval *argv = JS_ARGV(cx, vp);
     JSObject *newwin = JS_ConstructObjectWithArguments(jcx,
        &window_class, 0, jwin, argc, argv);
-    *rval = OBJECT_TO_JSVAL(newwin);
+    JS_SET_RVAL(cx, vp, OBJECT_TO_JSVAL(newwin));
+    return JS_TRUE;
 }				/* win_open */
 
 /* for window.focus etc */
 static JSBool
-nullFunction(JSContext * cx, JSObject * obj, uintN argc, jsval * argv,
-   jsval * rval)
+nullFunction(JSContext * cx, uintN argc, jsval * vp)
 {
+    JS_SET_RVAL(cx, vp, JSVAL_VOID);
     return JS_TRUE;
 }				/* nullFunction */
 
 static JSBool
-falseFunction(JSContext * cx, JSObject * obj, uintN argc, jsval * argv,
-   jsval * rval)
+falseFunction(JSContext * cx, uintN argc, jsval * vp)
 {
-    *rval = JSVAL_FALSE;
+    JS_SET_RVAL(cx, vp, JSVAL_FALSE);
     return JS_TRUE;
 }				/* falseFunction */
 
 static JSBool
-trueFunction(JSContext * cx, JSObject * obj, uintN argc, jsval * argv,
-   jsval * rval)
+trueFunction(JSContext * cx, uintN argc, jsval * vp)
 {
-    *rval = JSVAL_TRUE;
+    JS_SET_RVAL(cx, vp, JSVAL_TRUE);
     return JS_TRUE;
 }				/* trueFunction */
 
 static JSBool
-setAttribute(JSContext * cx, JSObject * obj, uintN argc, jsval * argv,
-   jsval * rval)
+setAttribute(JSContext * cx, uintN argc, jsval * vp)
 {
+    JSObject *this = JS_THIS_OBJECT(cx, vp);
+    jsval *argv = JS_ARGV(cx, vp);
     if(argc != 2 || !JSVAL_IS_STRING(argv[0])) {
 	JS_ReportError(jcx, "unexpected arguments to setAttribute()");
     } else {
 	const char *prop = stringize(argv[0]);
-	JS_DefineProperty(jcx, obj, prop, argv[1], NULL, NULL, PROP_FIXED);
+	JS_DefineProperty(jcx, this, prop, argv[1], NULL, NULL, PROP_FIXED);
     }
+    JS_SET_RVAL(cx, vp, JSVAL_VOID);
     return JS_TRUE;
 }				/* setAttribute */
 
 static JSBool
-appendChild(JSContext * cx, JSObject * obj, uintN argc, jsval * argv,
-   jsval * rval)
+appendChild(JSContext * cx, uintN argc, jsval * vp)
 {
     JSObject *elar;		/* elements array */
     jsuint length;
     jsval v;
-    JS_GetProperty(jcx, obj, "elements", &v);
+    jsval *argv = JS_ARGV(cx, vp);
+    JSObject *this = JS_THIS_OBJECT(cx, vp);
+    JS_GetProperty(jcx, this, "elements", &v);
     elar = JSVAL_TO_OBJECT(v);
     JS_GetArrayLength(jcx, elar, &length);
     JS_DefineElement(jcx, elar, length,
@@ -195,49 +265,50 @@ appendChild(JSContext * cx, JSObject * obj, uintN argc, jsval * argv,
 }				/* appendChild */
 
 static JSBool
-win_close(JSContext * cx, JSObject * obj, uintN argc, jsval * argv,
-   jsval * rval)
+win_close(JSContext * cx, uintN argc, jsval * vp)
 {
 /* It's too confusing to just close the window */
     i_puts(MSG_PageDone);
     cw->jsdead = true;
+    JS_SET_RVAL(cx, vp, JSVAL_VOID);
     return JS_TRUE;
 }				/* win_close */
 
 static JSBool
-win_alert(JSContext * cx, JSObject * obj, uintN argc, jsval * argv,
-   jsval * rval)
+win_alert(JSContext * cx, uintN argc, jsval * vp)
 {
-    const char *msg;
+    jsval *argv = JS_ARGV(cx, vp);
+    char *msg = NULL;
     JSString *str;
     if(argc > 0 && (str = JS_ValueToString(jcx, argv[0]))) {
-	msg = JS_GetStringBytes(str);
+	msg = transcode_get_js_bytes(str);
     }
-    if(msg)
+    if(msg) {
 	puts(msg);
+	nzFree(msg);
+    }
+    JS_SET_RVAL(cx, vp, JSVAL_VOID);
     return JS_TRUE;
 }				/* win_alert */
 
 static JSBool
-win_prompt(JSContext * cx, JSObject * obj, uintN argc, jsval * argv,
-   jsval * rval)
+win_prompt(JSContext * cx, uintN argc, jsval * vp)
 {
-    const char *msg = 0;
-    const char *answer = 0;
+    jsval *argv = JS_ARGV(cx, vp);
+    char *msg = EMPTYSTRING;
+    char *answer = EMPTYSTRING;
     JSString *str;
     char inbuf[80];
     char *s;
     char c;
 
     if(argc > 0 && (str = JS_ValueToString(jcx, argv[0]))) {
-	msg = JS_GetStringBytes(str);
+	msg = transcode_get_js_bytes(str);
     }
     if(argc > 1 && (str = JS_ValueToString(jcx, argv[1]))) {
-	answer = JS_GetStringBytes(str);
+	answer = transcode_get_js_bytes(str);
     }
 
-    if(!msg)
-	msg = "";
     printf("%s", msg);
 /* If it doesn't end in space or question mark, print a colon */
     c = 'x';
@@ -256,31 +327,28 @@ win_prompt(JSContext * cx, JSObject * obj, uintN argc, jsval * argv,
     s = inbuf + strlen(inbuf);
     if(s > inbuf && s[-1] == '\n')
 	*--s = 0;
-    if(inbuf[0])
+    if(inbuf[0]) {
+	nzFree(answer);		/* Don't need the default answer anymore. */
 	answer = inbuf;
-    if(!answer)
-	answer = "";
-    *rval = STRING_TO_JSVAL(JS_NewStringCopyZ(jcx, answer));
+    }
+    JS_SET_RVAL(cx, vp, STRING_TO_JSVAL(our_JS_NewStringCopyZ(jcx, answer)));
     return JS_TRUE;
 }				/* win_prompt */
 
 static JSBool
-win_confirm(JSContext * cx, JSObject * obj, uintN argc, jsval * argv,
-   jsval * rval)
+win_confirm(JSContext * cx, uintN argc, jsval * vp)
 {
-    const char *msg = 0;
+    jsval *argv = JS_ARGV(cx, vp);
+    char *msg = EMPTYSTRING;
     JSString *str;
     char inbuf[80];
-    char *s;
     char c;
     bool first = true;
 
     if(argc > 0 && (str = JS_ValueToString(jcx, argv[0]))) {
-	msg = JS_GetStringBytes(str);
+	msg = transcode_get_js_bytes(str);
     }
 
-    if(!msg)
-	msg = "";
     while(true) {
 	printf("%s", msg);
 	c = 'x';
@@ -304,16 +372,17 @@ win_confirm(JSContext * cx, JSObject * obj, uintN argc, jsval * argv,
 
     c = tolower(c);
     if(c == 'y')
-	*rval = JSVAL_TRUE;
+	JS_SET_RVAL(cx, vp, JSVAL_TRUE);
     else
-	*rval = JSVAL_FALSE;
+	JS_SET_RVAL(cx, vp, JSVAL_FALSE);
+    nzFree(msg);
     return JS_TRUE;
 }				/* win_confirm */
 
 static JSClass timer_class = {
     "Timer",
     JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
 };
 
@@ -328,11 +397,13 @@ setTimeout(uintN argc, jsval * argv, bool isInterval)
     char fname[48];		/* function name */
     const char *fstr;		/* function string */
     const char *methname = (isInterval ? "setInterval" : "setTimeout");
+    char *allocatedName = NULL;
+    char *s = NULL;
 
     if(!parsePage) {
 	JS_ReportError(jcx,
 	   "cannot use %s() to delay the execution of a function", methname);
-	return JSVAL_NULL;
+	return NULL;
     }
 
     if(argc != 2 || !JSVAL_IS_INT(argv[1]))
@@ -355,7 +426,10 @@ setTimeout(uintN argc, jsval * argv, bool isInterval)
 	if(fo) {
 /* Extract the function name, which requires several steps */
 	    JSFunction *f = JS_ValueToFunction(jcx, OBJECT_TO_JSVAL(fo));
-	    const char *s = JS_GetFunctionName(f);
+	    JSString *jss = JS_GetFunctionId(f);
+	    if(jss)
+		allocatedName = our_JSEncodeString(jss);
+	    s = allocatedName;
 /* Remember that unnamed functions are named anonymous. */
 	    if(!s || !*s || stringEqual(s, "anonymous"))
 		s = "javascript";
@@ -363,6 +437,7 @@ setTimeout(uintN argc, jsval * argv, bool isInterval)
 	    if(len > sizeof (fname) - 4)
 		len = sizeof (fname) - 4;
 	    strncpy(fname, s, len);
+	    nzFree(allocatedName);
 	    fname[len] = 0;
 	    strcat(fname, "()");
 	    fstr = fname;
@@ -380,41 +455,43 @@ setTimeout(uintN argc, jsval * argv, bool isInterval)
 
   badarg:
     JS_ReportError(jcx, "invalid arguments to %s()", methname);
-    return JSVAL_NULL;
+    return NULL;
 }				/* setTimeout */
 
 static JSBool
-win_sto(JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * rval)
+win_sto(JSContext * cx, uintN argc, jsval * vp)
 {
-    *rval = OBJECT_TO_JSVAL(setTimeout(argc, argv, false));
+    jsval *argv = JS_ARGV(cx, vp);
+    JS_SET_RVAL(cx, vp, OBJECT_TO_JSVAL(setTimeout(argc, argv, false)));
     return JS_TRUE;
 }				/* win_sto */
 
 static JSBool
-win_intv(JSContext * cx, JSObject * obj, uintN argc, jsval * argv, jsval * rval)
+win_intv(JSContext * cx, uintN argc, jsval * vp)
 {
-    *rval = OBJECT_TO_JSVAL(setTimeout(argc, argv, true));
+    jsval *argv = JS_ARGV(cx, vp);
+    JS_SET_RVAL(cx, vp, OBJECT_TO_JSVAL(setTimeout(argc, argv, true)));
     return JS_TRUE;
 }				/* win_intv */
 
 static JSFunctionSpec window_methods[] = {
-    {"alert", win_alert, 1, 0, 0},
-    {"prompt", win_prompt, 2, 0, 0},
-    {"confirm", win_confirm, 1, 0, 0},
-    {"setTimeout", win_sto, 2, 0, 0},
-    {"setInterval", win_intv, 2, 0, 0},
-    {"open", win_open, 3, 0, 0},
-    {"close", win_close, 0, 0, 0},
-    {"focus", nullFunction, 0, 0, 0},
-    {"blur", nullFunction, 0, 0, 0},
-    {"scroll", nullFunction, 0, 0, 0},
+    {"alert", win_alert, 1, 0},
+    {"prompt", win_prompt, 2, 0},
+    {"confirm", win_confirm, 1, 0},
+    {"setTimeout", win_sto, 2, 0},
+    {"setInterval", win_intv, 2, 0},
+    {"open", win_open, 3, 0},
+    {"close", win_close, 0, 0},
+    {"focus", nullFunction, 0, 0},
+    {"blur", nullFunction, 0, 0},
+    {"scroll", nullFunction, 0, 0},
     {0}
 };
 
 static JSClass doc_class = {
     "Document",
     JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
 };
 
@@ -432,28 +509,31 @@ static void
 dwrite1(uintN argc, jsval * argv, bool newline)
 {
     int i;
-    const char *msg;
+    char *msg;
     JSString *str;
     for(i = 0; i < argc; ++i) {
 	if((str = JS_ValueToString(jcx, argv[i])) &&
-	   (msg = JS_GetStringBytes(str)))
+	   (msg = transcode_get_js_bytes(str))) {
 	    dwrite2(msg);
-/* I assume I don't have to free msg?? */
+	    nzFree(msg);
+	}
     }
     if(newline)
 	dwrite2("\n");
 }				/* dwrite1 */
 
 static JSBool
-doc_write(JSContext * cx, JSObject * obj, uintN argc, jsval * argv,
-   jsval * rval)
+doc_write(JSContext * cx, uintN argc, jsval * vp)
 {
+    jsval *argv = JS_ARGV(cx, vp);
     dwrite1(argc, argv, false);
+    JS_SET_RVAL(cx, vp, JSVAL_VOID);
     return JS_TRUE;
 }				/* doc_write */
 
 static JSBool
-setter_innerHTML(JSContext * cx, JSObject * obj, jsval id, jsval * vp)
+setter_innerHTML(JSContext * cx, JSObject * obj, jsid id, JSBool strict,
+   jsval * vp)
 {
     const char *s = stringize(*vp);
     if(s && strlen(s)) {
@@ -467,108 +547,113 @@ setter_innerHTML(JSContext * cx, JSObject * obj, jsval id, jsval * vp)
 }				/* setter_innerHTML */
 
 static JSBool
-setter_innerText(JSContext * cx, JSObject * obj, jsval id, jsval * vp)
+setter_innerText(JSContext * cx, JSObject * obj, jsid id, JSBool strict,
+   jsval * vp)
 {
     jsval v = *vp;
-    const char *s;
+    char *s;
     if(!JSVAL_IS_STRING(v))
 	return JS_FALSE;
-    s = JS_GetStringBytes(JSVAL_TO_STRING(v));
+    s = our_JSEncodeString(JSVAL_TO_STRING(v));
+    nzFree(s);
     i_puts(MSG_InnerText);
 /* The string has already been updated in the object. */
     return JS_TRUE;
 }				/* setter_innerText */
 
 static JSBool
-doc_writeln(JSContext * cx, JSObject * obj, uintN argc, jsval * argv,
-   jsval * rval)
+doc_writeln(JSContext * cx, uintN argc, jsval * vp)
 {
+    jsval *argv = JS_ARGV(cx, vp);
     dwrite1(argc, argv, true);
+    JS_SET_RVAL(cx, vp, JSVAL_VOID);
     return JS_TRUE;
 }				/* doc_writeln */
 
 static JSFunctionSpec doc_methods[] = {
-    {"focus", nullFunction, 0, 0, 0},
-    {"blur", nullFunction, 0, 0, 0},
-    {"open", nullFunction, 0, 0, 0},
-    {"close", nullFunction, 0, 0, 0},
-    {"write", doc_write, 0, 0, 0},
-    {"writeln", doc_writeln, 0, 0, 0},
+    {"focus", nullFunction, 0, 0},
+    {"blur", nullFunction, 0, 0},
+    {"open", nullFunction, 0, 0},
+    {"close", nullFunction, 0, 0},
+    {"write", doc_write, 0, 0},
+    {"writeln", doc_writeln, 0, 0},
     {0}
 };
 
 static JSClass element_class = {
     "Element",
     JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
 };
 
 static JSFunctionSpec element_methods[] = {
-    {"focus", nullFunction, 0, 0, 0},
-    {"blur", nullFunction, 0, 0, 0},
+    {"focus", nullFunction, 0, 0},
+    {"blur", nullFunction, 0, 0},
     {0}
 };
 
 static JSClass form_class = {
     "Form",
     JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
 };
 
 static JSBool
-form_submit(JSContext * cx, JSObject * obj, uintN argc, jsval * argv,
-   jsval * rval)
+form_submit(JSContext * cx, uintN argc, jsval * vp)
 {
-    javaSubmitsForm(obj, false);
+    JSObject *this = JS_THIS_OBJECT(cx, vp);
+    javaSubmitsForm(this, false);
+    JS_SET_RVAL(cx, vp, JSVAL_VOID);
     return JS_TRUE;
 }				/* form_submit */
 
 static JSBool
-form_reset(JSContext * cx, JSObject * obj, uintN argc, jsval * argv,
-   jsval * rval)
+form_reset(JSContext * cx, uintN argc, jsval * vp)
 {
-    javaSubmitsForm(obj, true);
+    JSObject *this = JS_THIS_OBJECT(cx, vp);
+    javaSubmitsForm(this, true);
+    JS_SET_RVAL(cx, vp, JSVAL_VOID);
     return JS_TRUE;
 }				/* form_reset */
 
 static JSFunctionSpec form_methods[] = {
-    {"submit", form_submit, 0, 0, 0},
-    {"reset", form_reset, 0, 0, 0},
+    {"submit", form_submit, 0, 0},
+    {"reset", form_reset, 0, 0},
     {0}
 };
 
 static JSClass body_class = {
     "Body",
     JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
 };
 
 static JSFunctionSpec body_methods[] = {
-    {"setAttribute", setAttribute, 2, 0, 0},
-    {"appendChild", appendChild, 1, 0, 0},
+    {"setAttribute", setAttribute, 2, 0},
+    {"appendChild", appendChild, 1, 0},
     {0}
 };
 
 static JSClass head_class = {
     "Head",
     JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
 };
 
 static JSFunctionSpec head_methods[] = {
-    {"setAttribute", setAttribute, 2, 0, 0},
-    {"appendChild", appendChild, 1, 0, 0},
+    {"setAttribute", setAttribute, 2, 0},
+    {"appendChild", appendChild, 1, 0},
     {0}
 };
 
 static JSClass meta_class = {
     "Meta",
     JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
 };
 
@@ -576,89 +661,89 @@ static JSClass meta_class = {
 static JSClass link_class = {
     "Link",
     JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
 };
 
 static JSFunctionSpec link_methods[] = {
-    {"setAttribute", setAttribute, 2, 0, 0},
+    {"setAttribute", setAttribute, 2, 0},
     {0}
 };
 
 static JSClass image_class = {
     "Image",
     JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
 };
 
 static JSClass frame_class = {
     "Frame",
     JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
 };
 
 static JSClass anchor_class = {
     "Anchor",
     JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
 };
 
 static JSClass table_class = {
     "Table",
     JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
 };
 
 static JSClass trow_class = {
     "Trow",
     JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
 };
 
 static JSClass cell_class = {
     "Cell",
     JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
 };
 
 static JSClass div_class = {
     "Div",
     JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
 };
 
 static JSClass span_class = {
     "Span",
     JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
 };
 
 static JSClass area_class = {
     "Area",
     JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
 };
 
 static JSClass option_class = {
     "Option",
     JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
+    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, JS_FinalizeStub,
 };
 
 struct DOMCLASS {
     JSClass *class;
     JSFunctionSpec *methods;
-      JSBool(*constructor) (JSContext *, JSObject *, uintN, jsval *, jsval *);
+    JSNative constructor;
     int nargs;
 };
 
@@ -760,14 +845,12 @@ createJavaContext(void)
     JS_SetOptions(jcx, JSOPTION_VAROBJFIX);
 
 /* Create the Window object, which is the global object in DOM. */
-    jwin = JS_NewObject(jcx, &window_class, NULL, NULL);
+//    jwin = JS_NewObject(jcx, &window_class, NULL, NULL);
+    jwin = JS_NewCompartmentAndGlobalObject(jcx, &window_class, NULL);
     if(!jwin)
 	i_printfExit(MSG_JavaWindowError);
     JS_InitClass(jcx, jwin, 0, &window_class, window_ctor, 3,
        NULL, window_methods, NULL, NULL);
-/* Ok, but the global object was created before the class,
- * so it doesn't have its methods yet. */
-    JS_DefineFunctions(jcx, jwin, window_methods);
 
 /* Math, Date, Number, String, etc */
     if(!JS_InitStandardClasses(jcx, jwin))
@@ -887,7 +970,7 @@ createJavaContext(void)
 /* For the name, how about the program without its options? */
 	len = strcspn(mt->program, " \t");
 	JS_DefineProperty(jcx, po, "name",
-	   STRING_TO_JSVAL(JS_NewStringCopyN(jcx, mt->program, len)),
+	   STRING_TO_JSVAL(our_JS_NewStringCopyN(jcx, mt->program, len)),
 	   0, 0, PROP_FIXED);
     }
 
@@ -953,12 +1036,12 @@ establish_innerHTML(void *jv, const char *start, const char *end, bool is_ta)
     if(!start)
 	start = end = EMPTYSTRING;
     JS_DefineProperty(jcx, obj, "innerHTML",
-       STRING_TO_JSVAL(JS_NewStringCopyN(jcx, start, end - start)),
+       STRING_TO_JSVAL(our_JS_NewStringCopyN(jcx, start, end - start)),
        NULL, (is_ta ? setter_innerText : setter_innerHTML),
        JSPROP_ENUMERATE | JSPROP_PERMANENT);
     if(is_ta) {
 	JS_DefineProperty(jcx, obj, "innerText",
-	   STRING_TO_JSVAL(JS_NewStringCopyN(jcx, start, end - start)),
+	   STRING_TO_JSVAL(our_JS_NewStringCopyN(jcx, start, end - start)),
 	   NULL, setter_innerText, JSPROP_ENUMERATE | JSPROP_PERMANENT);
     }
 
@@ -1141,13 +1224,17 @@ Yeah, it makes my head spin too.
     if(symname)
 	establish_property_string(v, "name", symname, true);
     if(idname) {
-/* v.id becomes idname, and idMaster.idname becomes v */
-	establish_property_string(v, "id", idname, true);
+/* v.id becomes idname, and idMaster.idname becomes v
+ * In case of forms, v.id should remain undefined.  So we can have
+ * a form field named "id". */
+	if(strcmp(classname, "Form") != 0)
+	    establish_property_string(v, "id", idname, true);
 	JS_GetProperty(jcx, jdoc, "idMaster", &listv);
 	master = JSVAL_TO_OBJECT(listv);
 	establish_property_object(master, idname, v);
     } else {
-	establish_property_string(v, "id", EMPTYSTRING, true);
+	if(strcmp(classname, "Form") != 0)
+	    establish_property_string(v, "id", EMPTYSTRING, true);
     }
 
     if(href && href_url) {
