@@ -13,13 +13,13 @@
 /* See eb.h for descriptive comments. */
 
 
-const char *version = "3.4.5";
+const char *version = "3.4.8";
 char *userAgents[10], *currentAgent, *currentReferrer;
 const char eol[] = "\r\n";
 char EMPTYSTRING[] = "";
 int debugLevel = 1;
 int webTimeout = 20, mailTimeout = 0;
-bool ismc, browseLocal, zapMail, unformatMail, passMail, errorExit;
+bool ismc, browseLocal, passMail, errorExit;
 bool isInteractive, inInput, listNA;
 volatile bool intFlag;
 int fileSize, maxFileSize = 50000000;
@@ -42,8 +42,9 @@ uchar dirWrite, endMarks;
 int context = 1;
 uchar linePending[MAXTTYLINE];
 char *changeFileName, *mailDir;
+char *mailUnread, *mailStash;
 char *addressFile, *ipbFile;
-char *home, *recycleBin, *configFile, *sigFile;
+char *home, *recycleBin, *configFile, *sigFile, *sigFileEnd;
 char *cookieFile, *spamCan;
 char *edbrowseTempFile, *edbrowseTempPDF, *edbrowseTempHTML;
 pst *textLines;
@@ -168,7 +169,7 @@ readConfigFile(void)
 	"adbook", "ipblack", "maildir", "agent",
 	"jar", "nojs", "spamcan",
 	"webtimer", "mailtimer", "certfile", "datasource", "proxy",
-	"linelength", 0
+	"linelength", "localizeweb", 0
     };
 
     if(!fileTypeByName(configFile, false))
@@ -365,7 +366,7 @@ readConfigFile(void)
 		    filters[n_filters].expire = exp;
 		    if(exp <= nowday) {
 			cfgmodify = true;
-			memcpy(cfglp, cfgnlp, cfgcopy + cfglen - cfgnlp);
+			memmove(cfglp, cfgnlp, cfgcopy + cfglen - cfgnlp);
 			cfglen -= (cfgnlp - cfglp);
 			cfgnlp = cfglp;
 			continue;
@@ -543,6 +544,14 @@ readConfigFile(void)
 	    mailDir = v;
 	    if(fileTypeByName(v, false) != 'd')
 		i_printfExit(MSG_ERBC_NotDir, v);
+	    mailUnread = allocMem(strlen(v) + 12);
+	    sprintf(mailUnread, "%s/unread", v);
+/* We need the unread directory, else we can't fetch mail. */
+/* Create it if it isn't there. */
+	    if(fileTypeByName(mailUnread, false) != 'd') {
+		if(mkdir(mailUnread, 0700))
+		    i_printfExit(MSG_ERBC_NotDir, mailUnread);
+	    }
 	    continue;
 
 	case 20:
@@ -616,6 +625,12 @@ readConfigFile(void)
 		displayLength = 80;
 	    continue;
 
+	case 30:		/* localizeweb */
+/* We should probably allow autodetection of language. */
+/* E.G., the keyword auto indicates that you want autodetection. */
+	    setHTTPLanguage(v);
+	    continue;
+
 	default:
 	    i_printfExit(MSG_ERBC_KeywordNYI, ln, s);
 	}			/* switch */
@@ -631,6 +646,11 @@ readConfigFile(void)
 	    continue;
 	}
 
+	if(stringEqual(s, "nofetch") && mailblock == 1) {
+	    act->nofetch = 1;
+	    continue;
+	}
+
 	if(*s == '\x82' && s[1] == 0) {
 	    if(mailblock == 1) {
 		++maxAccount;
@@ -982,6 +1002,7 @@ main(int argc, char **argv)
 {
     int cx, account;
     bool rc, doConfig = true;
+    bool dofetch = false, domail = false;
 
 /* In case this is being piped over to a synthesizer, or whatever. */
     if(fileTypeByHandle(fileno(stdout)) != 'f')
@@ -990,6 +1011,7 @@ main(int argc, char **argv)
     selectLanguage();
 
     ttySaveSettings();
+    initializeReadline();
 
 /* Let's everybody use my malloc and free routines */
     pcre_malloc = allocMem;
@@ -1012,6 +1034,7 @@ main(int argc, char **argv)
 
     recycleBin = allocMem(strlen(home) + 8);
     sprintf(recycleBin, "%s/.Trash", home);
+
     edbrowseTempFile = allocMem(strlen(recycleBin) + 8 + 6);
 /* The extra 6 is for the suffix */
     sprintf(edbrowseTempFile, "%s/eb_tmp", recycleBin);
@@ -1019,15 +1042,30 @@ main(int argc, char **argv)
     sprintf(edbrowseTempPDF, "%s/eb_pdf", recycleBin);
     edbrowseTempHTML = allocMem(strlen(recycleBin) + 13);
     sprintf(edbrowseTempHTML, "%s/eb_pdf.html", recycleBin);
+
     if(fileTypeByName(recycleBin, false) != 'd') {
 	if(mkdir(recycleBin, 0700)) {
+/* Don't want to abort here; we might be on a readonly filesystem.
+ * Don't have a Trash directory and can't creat one; yet we should move on. */
 	    free(recycleBin);
 	    recycleBin = 0;
 	}
     }
 
-    sigFile = allocMem(strlen(home) + 12);
+    if(recycleBin) {
+	mailStash = allocMem(strlen(recycleBin) + 12);
+	sprintf(mailStash, "%s/rawmail", recycleBin);
+	if(fileTypeByName(mailStash, false) != 'd') {
+	    if(mkdir(mailStash, 0700)) {
+		free(mailStash);
+		mailStash = 0;
+	    }
+	}
+    }
+
+    sigFile = allocMem(strlen(home) + 20);
     sprintf(sigFile, "%s/.signature", home);
+    sigFileEnd = sigFile + strlen(sigFile);
 
     {
 	static char agent0[32] = "edbrowse/";
@@ -1054,41 +1092,57 @@ main(int argc, char **argv)
     for(; argc && argv[0][0] == '-'; ++argv, --argc) {
 	char *s = *argv;
 	++s;
+
 	if(stringEqual(s, "v")) {
 	    puts(version);
 	    exit(0);
 	}
+
 	if(stringEqual(s, "d")) {
 	    debugLevel = 4;
 	    continue;
 	}
+
 	if(*s == 'd' && isdigitByte(s[1]) && !s[2]) {
 	    debugLevel = s[1] - '0';
 	    continue;
 	}
+
 	if(stringEqual(s, "e")) {
 	    errorExit = true;
 	    continue;
 	}
-	if(*s == 'u')
-	    ++s, unformatMail = true;
+
 	if(*s == 'p')
 	    ++s, passMail = true;
-	if(*s == 'm' && isdigitByte(s[1])) {
+
+	if(*s == 'm' || *s == 'f') {
 	    if(!maxAccount)
 		i_printfExit(MSG_NoMailAcc);
-	    account = strtol(s + 1, &s, 10);
-	    if(account == 0 || account > maxAccount)
-		i_printfExit(MSG_BadAccNb, maxAccount);
+	    if(*s == 'f') {
+		account = 0;
+		dofetch = true;
+		++s;
+		if(*s == 'm')
+		    domail = true, ++s;
+	    } else {
+		domail = true;
+		++s;
+	    }
+	    if(isdigitByte(*s)) {
+		account = strtol(s, &s, 10);
+		if(account == 0 || account > maxAccount)
+		    i_printfExit(MSG_BadAccNb, maxAccount);
+	    }
 	    if(!*s) {
 		ismc = true;	/* running as a mail client */
 		allowJS = false;	/* no javascript in mail client */
-		++argv, --argc;	/* we're going to break out */
-		if(argc && stringEqual(argv[0], "-Zap"))
-		    ++argv, --argc, zapMail = true;
-		break;
+		++argv, --argc;
+		if(!argc || !dofetch)
+		    break;
 	    }
 	}
+
 	i_printfExit(MSG_Usage);
     }				/* options */
 
@@ -1114,8 +1168,46 @@ main(int argc, char **argv)
 	char **reclist, **atlist;
 	char *s, *body;
 	int nat, nalt, nrec;
-	if(!argc)
-	    fetchMail(account);
+
+	if(!argc) {
+/* This is fetch / read mode */
+	    if(dofetch) {
+		int nfetch = 0;
+		if(account) {
+		    nfetch = fetchMail(account);
+		} else {
+		    int i, j;
+		    for(i = 1; i <= maxAccount; ++i) {
+/* did we set this to nofetch in the config file? */
+			if(accounts[i - 1].nofetch)
+			    continue;
+/* don't fetch from a different account that has the same host an dlogin */
+			for(j = 1; j < i; ++j)
+			    if(stringEqual(accounts[i - 1].inurl,
+			       accounts[j - 1].inurl) &&
+			       stringEqual(accounts[i - 1].login,
+			       accounts[j - 1].login))
+				break;
+			if(j == i)
+			    nfetch += fetchMail(i);
+		    }
+		}
+		if(!domail) {
+		    if(nfetch)
+			i_printf(MSG_MessagesX, nfetch);
+		    else
+			i_puts(MSG_NoMail);
+		}
+	    }
+
+	    if(domail) {
+		scanMail();
+	    }
+
+	    exit(0);
+	}
+
+/* now in sendmail mode */
 	if(argc == 1)
 	    i_printfExit(MSG_MinOneRec);
 /* I don't know that argv[argc] is 0, or that I can set it to 0,
@@ -1127,17 +1219,17 @@ main(int argc, char **argv)
 		break;
 	    if(*s == '-')
 		++nalt;
-	    strcpy(s, s + 1);
+	    strmove(s, s + 1);
 	}
 	atlist = argv + argc - nat - 1;
 	if(atlist <= argv)
 	    i_printfExit(MSG_MinOneRecBefAtt);
 	body = *atlist;
 	if(nat)
-	    memcpy(atlist, atlist + 1, sizeof (char *) * nat);
+	    memmove(atlist, atlist + 1, sizeof (char *) * nat);
 	atlist[nat] = 0;
 	nrec = atlist - argv;
-	memcpy(reclist, reclist + 1, sizeof (char *) * nrec);
+	memmove(reclist, reclist + 1, sizeof (char *) * nrec);
 	atlist[-1] = 0;
 	if(sendMail(account, (const char **)reclist, body, 1,
 	   (const char **)atlist, 0, nalt, true))
@@ -1280,7 +1372,7 @@ runEbFunction(const char *line)
 	    goto fail;
 	}
     for(j = 0; ebScript[j]; ++j)
-	if(stringEqual(linecopy, ebScriptName[j] + 1))
+	if(stringEqualCI(linecopy, ebScriptName[j] + 1))
 	    break;
     if(!ebScript[j]) {
 	setError(MSG_NoSuchFunction, linecopy);
