@@ -19,6 +19,7 @@ static bool doSignature;
 static bool ssl_on;
 static const char *mailhost;
 static char subjectLine[200];
+static int mailAccount;
 
 static struct ALIAS {
     char name[16];
@@ -61,6 +62,7 @@ loadAddressBook(void)
 		setError(MSG_ABNoAlias, ln);
 	      freefail:
 		nzFree(buf);
+		nads = 0;
 		return false;
 	    }
 	    while(t[-1] == ' ' || t[-1] == '\t')
@@ -302,7 +304,6 @@ void
 mailClose(void)
 {
     mailPutLine("quit\r\n", false);
-    endhostent();
 /* the other side has to have time to process the quit command, before we simply hang up. */
     usleep(400000);
     if(ssl_on)
@@ -593,11 +594,17 @@ encodeAttachment(const char *file, int ismail, bool webform,
 		break;
 	buflen -= (v - buf);
 	if(buflen)
-	    memcpy(buf, v, buflen);
+	    memmove(buf, v, buflen);
 	buf[buflen] = 0;
 
 	if(doSignature) {	/* Append .signature file. */
+/* Try account specific .signature file, then fall back to .signature */
+	    sprintf(sigFileEnd, "%d", mailAccount);
 	    c = fileTypeByName(sigFile, false);
+	    if(!c) {
+		*sigFileEnd = 0;
+		c = fileTypeByName(sigFile, false);
+	    }
 	    if(c != 0) {
 		int fd, n;
 		if(c != 'f') {
@@ -867,6 +874,7 @@ sendMail(int account, const char **recipients, const char *body,
 
     if(!validAccount(account))
 	return false;
+    mailAccount = account;
     localMail = accounts + localAccount - 1;
 
     a = accounts + account - 1;
