@@ -126,7 +126,7 @@ stripWhite(char *s)
     char *u;
     skipWhite(&t);
     if(t > s)
-	strcpy(s, t);
+	strmove(s, t);
     u = s + strlen(s);
     while(u > s && isspaceByte(u[-1]))
 	--u;
@@ -157,6 +157,13 @@ spaceCrunch(char *s, bool onespace, bool unprint)
     s[j] = 0;
 }				/* spaceCrunch */
 
+/* Like strcpy, but able to cope with overlapping strings. */
+char *
+strmove(char *dest, const char *src)
+{
+    return memmove(dest, src, strlen(src) + 1);
+}				/* strmove */
+
 /* OO has a lot of unnecessary overhead, and a few inconveniences,
  * but I really miss it right now.  The following
  * routines make up for the lack of simple string concatenation in C.
@@ -308,7 +315,7 @@ leftClipString(char *s)
 	if(!isspace(*t))
 	    break;
     if(t > s)
-	strcpy(s, t);
+	strmove(s, t);
 }				/* leftClipString */
 
 /* shift string one to the right */
@@ -392,6 +399,8 @@ isSQL(const char *s)
     char c;
     const char *c1 = 0, *c2 = 0;
     c = *s;
+    if(isURL(s))
+	return false;
     if(!isalphaByte(c))
 	goto no;
     for(++s; c = *s; ++s) {
@@ -674,6 +683,7 @@ clonePstring(pst s)
     return t;
 }				/* clonePstring */
 
+// Strings are assumed distinct, hence the use of memcpy.
 void
 copyPstring(pst s, const pst t)
 {
@@ -681,12 +691,52 @@ copyPstring(pst s, const pst t)
     memcpy(s, t, len);
 }				/* copyPstring */
 
+/*
+ * fdIntoMemory reads data from a file descriptor, until EOF is reached.
+ * It works even if we don't know the size beforehand.
+ * We can now use it to read /proc files, pipes, and stdin.
+ * This solves an outstanding issue, and it is needed for forthcoming
+ * functionality, such as edpager.
+ */
+bool
+fdIntoMemory(int fd, char **data, int *len)
+{
+    int length, n;
+    const int CHUNKSIZE = 8192;
+    char *chunk, *buf;
+
+    chunk = allocZeroMem(CHUNKSIZE);
+    buf = initString(&length);
+
+    n = 0;
+    do {
+	n = read(fd, chunk, CHUNKSIZE);
+	if(n < 0) {
+	    nzFree(buf);
+	    nzFree(chunk);
+	    *data = EMPTYSTRING;
+	    *len = 0;
+	    setError(MSG_NoRead, "file descriptor");
+	    return false;
+	}
+
+	if(n > 0)
+	    stringAndBytes(&buf, &length, chunk, n);
+    } while(n != 0);
+
+    nzFree(chunk);
+    buf = reallocMem(buf, length + 2);
+    *data = buf;
+    *len = length;
+    return true;
+}				/* fdIntoMemory */
+
 bool
 fileIntoMemory(const char *filename, char **data, int *len)
 {
-    int length, n, fh;
-    char *buf;
+    int fh;
     char ftype = fileTypeByName(filename, false);
+    bool ret;
     if(ftype && ftype != 'f') {
 	setError(MSG_RegularFile, filename);
 	return false;
@@ -696,31 +746,13 @@ fileIntoMemory(const char *filename, char **data, int *len)
 	setError(MSG_NoOpen, filename);
 	return false;
     }
-    length = fileSizeByName(filename);
-    if(length < 0) {
-	close(fh);
-	return false;
-    }				/* should never hapen */
-#if 0
-    if(length > maxFileSize) {
-	setError(MSG_LargeFile);
-	close(fh);
-	return false;
-    }
-#endif
-    buf = allocMem(length + 2);
-    n = 0;
-    if(length)
-	n = read(fh, buf, length);
-    close(fh);			/* don't need that any more */
-    if(n < length) {
+
+    ret = fdIntoMemory(fh, data, len);
+    if(ret == false)
 	setError(MSG_NoRead2, filename);
-	free(buf);
-	return false;
-    }
-    *data = buf;
-    *len = length;
-    return true;
+
+    close(fh);
+    return ret;
 }				/* fileIntoMemory */
 
 /* inverse of the above */
